function [pks,locs_y,locs_x]=peaks2(data,varargin)

if ~ismatrix(data)
    error('Only 1D (vectors) or 2D (matrices) data accepted.');
end

ispeak=false(size(data)); 
isgreater=true(size(data)); 

ispeak=([true(size(data,1),1) [data(:,2:end)>data(:,1:end-1)]]); 
%
% 2nd neighbor at the top-left:
isgreater(2:end,2:end)=(data(2:end,2:end)>data(1:end-1,1:end-1)); 
ispeak=ispeak&isgreater; % time to update the peak array.
%
% 3rd neighbor immediatelly at the top:
ispeak=ispeak&([true(1,size(data,2)); (data(2:end,:)>data(1:end-1,:))]); 
%
% 4th neighbor at the top-right:
isgreater=true(size(data)); % rebuild a fresh array for comparison...
isgreater(2:end,1:end-1)=(data(2:end,1:end-1)>data(1:end-1,2:end));
ispeak=ispeak&isgreater;
%
% 5th neighbor immediatelly to the right:
ispeak=ispeak&([(data(:,1:end-1)>data(:,2:end)) true(size(data,1),1)]); % once again, for this case, we can update the peak array directly.
%
% 6th neighbor to the bottom-right:
isgreater=true(size(data));
isgreater(1:end-1,1:end-1)=(data(1:end-1,1:end-1)>data(2:end,2:end));
ispeak=ispeak&isgreater;
%
% 7th neighbor immediatelly at the bottom:
ispeak=ispeak&([(data(1:end-1,:)>data(2:end,:)); true(1,size(data,2))]); % once again, for this case, we can update the peak array directly.
%
% 8th neighbor at the bottom-left:
isgreater=true(size(data));
isgreater(1:end-1,2:end)=(data(1:end-1,2:end)>data(2:end,1:end-1));
ispeak=ispeak&isgreater;

clear isgreater

locs=find(ispeak); 
pks=data(locs);

[locs_y,locs_x]=ind2sub(size(data),locs);

if isempty(varargin)
    return
end

if mod(length(varargin),2)~=0
    warning('Optional name-value parameters should come in pairs. Something is missing. Peak search will proceed with default values.');
    return
end

params=varargin(1:2:end);
vals=varargin(2:2:end);

ischarparam=cellfun(@ischar,params);
if any(~ischarparam)
    warning('Some parameters are not written as characters and not recognisable. They should always come is name(char)-value pairs. Peak search will proceed with default values.');
    return
end; clear ischarparam varargin

%--------------------------------------------------------------------------
% No go over all supplied parameters and check the found peaks accordingly.

% 1. MinPeakHeight - absolute minimum value for a peak:
isparm=find(cellfun(@(x)isequal(x,'MinPeakHeight'),params),1);
if ~isempty(isparm)
    % Locate which peaks satisfy this condition:
    suitable=(pks>=vals{isparm});
    % ...and only keep those:
    pks=pks(suitable);
    locs=locs(suitable);
    clear suitable
end

% 2. Threshold - peak must be greater than its neighbours by this value.
isparm=find(cellfun(@(x)isequal(x,'Threshold'),params),1);
if ~isempty(isparm)
    % For this, we will need to convert linear indices to array indices:
    [row_y,col_x]=ind2sub(size(data),locs);
    % These will be the original indices.
    
    % This is how array coordinates would change relatively around each
    % peak (y,x):
    % (-1,-1)  (-1,0)  (-1,+1)
    % ( 0,-1)  ( 0,0)  ( 0,+1)
    % (+1,-1)  (+1,0)  (+1,+1)
    % ...turned into vectors disregarding the (0,0), the center data point:
    delta_y=[-1 -1 -1 0 0 +1 +1 +1];
    delta_x=[-1 0 +1 -1 +1 -1 0 +1];
    % Let's add these deltas to the detected peak positions to get the
    % coordinates of their immediate neighbors:
    neighbor_locs_y=row_y+delta_y;
    neighbor_locs_x=col_x+delta_x; clear row_y col_x delta_x delta_y
    % ...don't forget to check for unrealistic indices beyond array
    % borders:
    neighbor_locs_y(neighbor_locs_y<1)=1;
    neighbor_locs_y(neighbor_locs_y>size(data,1))=size(data,1);
    neighbor_locs_x(neighbor_locs_x<1)=1;
    neighbor_locs_x(neighbor_locs_x>size(data,2))=size(data,2);
    % ...convert to linear indices:
    neighbor_locs=sub2ind(size(data),neighbor_locs_y,neighbor_locs_x);
    clear neighbor_locs_y neighbor_locs_x
    
    % So we have neighbor values by now. Are our peaks higher than those by
    % the set Threshold value?
    suitable=(data(locs)-vals{isparm}>=data(neighbor_locs));
    % Now check for those cases when by mistake (earlier step for checking
    % for indices beyond array boundaries) a peak itself is taken as a
    % neighbor as well:
    suitable(data(neighbor_locs)==data(locs))=true;
    
    % Only keep those is they are greater than ALL neighbors (in other
    % words, those elements where NONE are lesser):
    suitable=~any(~suitable,2); clear neighbor_locs
    
    % Final step - locate suitable element indices:
    suitable=find(suitable);
    
    % That's it, modify the output array:
    pks=pks(suitable);
    locs=locs(suitable); clear suitable
end

% 3. 'MinPeakDistance'
isparm=find(cellfun(@(x)isequal(x,'MinPeakDistance'),params),1);
if ~isempty(isparm)
    
    % First, sort the peaks in order of amplitude:
    [pks_sorted,idx]=sort(pks,'descend');
    locs_sorted=locs(idx); clear idx

    % The flow is as follows: start from the highest peak and discard any
    % other peaks closer than the CARTESIAN MinPeakDistance (that is, the
    % CIRCLE around the peak is going to be checked); then continue until
    % the whole list (updated iteratively as items might get removed) has
    % been checked.
    
    % Convert locations to array indices:
    [row_y,col_x]=ind2sub(size(data),locs_sorted);

    % Start from the highest peak:
    this_peak=1;
    while this_peak<(length(pks_sorted)+1)

        % Cartesian distances to ALL its remaining & yet unchecked neighbors
        % (including itself):
        dist=sqrt((row_y-row_y(this_peak)).^2+(col_x-col_x(this_peak)).^2);
        % Now simply check which neighbors are WITHIN the MinPeakDistance
        % BUT also nonzero (the peak should not be compared to itself):
        within=( (dist<=vals{isparm}) & (dist~=0) );
        % ...and delete those entries satisfying the condition:
        pks_sorted(within)=[];
        locs_sorted(within)=[];
        row_y(within)=[]; col_x(within)=[];

        % Update the peak counter:
        this_peak=this_peak+1;

    end; clear this_peak within dist row_y col_x

    % Update the peak location and value list:
    pks=pks_sorted;
    locs=locs_sorted;
end

[locs_y,locs_x]=ind2sub(size(data),locs);

end